UniformFloat

Struct UniformFloat 

pub struct UniformFloat<X> { /* private fields */ }
The back-end implementing UniformSampler for floating-point types.

Unless you are implementing UniformSampler for your own type, this type should not be used directly, use Uniform instead.

Implementation notes

UniformFloat implementations convert RNG output to a float in the range [1, 2) via transmutation, map this to [0, 1), then scale and translate to the desired range. Values produced this way have what equals 23 bits of random digits for an f32 and 52 for an f64.

Bias and range errors

Bias may be expected within the least-significant bit of the significand. It is not guaranteed that exclusive limits of a range are respected; i.e. when sampling the range [a, b) it is not guaranteed that b is never sampled.