UniformInt

Struct UniformInt 

pub struct UniformInt<X> { /* private fields */ }
The back-end implementing UniformSampler for integer types.

Unless you are implementing UniformSampler for your own type, this type should not be used directly, use Uniform instead.

Implementation notes

For simplicity, we use the same generic struct UniformInt<X> for all integer types X. This gives us only one field type, X; to store unsigned values of this size, we take use the fact that these conversions are no-ops.

For a closed range, the number of possible numbers we should generate is range = (high - low + 1). To avoid bias, we must ensure that the size of our sample space, zone, is a multiple of range; other values must be rejected (by replacing with a new random sample).

As a special case, we use range = 0 to represent the full range of the result type (i.e. for new_inclusive($ty::MIN, $ty::MAX)).

The optimum zone is the largest product of range which fits in our (unsigned) target type. We calculate this by calculating how many numbers we must reject: reject = (MAX + 1) % range = (MAX - range + 1) % range. Any (large) product of range will suffice, thus in sample_single we multiply by a power of 2 via bit-shifting (faster but may cause more rejections).

The smallest integer PRNGs generate is u32. For 8- and 16-bit outputs we use u32 for our zone and samples (because it’s not slower and because it reduces the chance of having to reject a sample). In this case we cannot store zone in the target type since it is too large, however we know ints_to_reject < range <= $uty::MAX.

An alternative to using a modulus is widening multiply: After a widening multiply by range, the result is in the high word. Then comparing the low word against zone makes sure our distribution is uniform.

Bias

Unless the unbiased feature flag is used, outputs may have a small bias. In the worst case, bias affects 1 in 2^n samples where n is 56 (i8 and u8), 48 (i16 and u16), 96 (i32 and u32), 64 (i64 and u64), 128 (i128 and u128).