Struct UniformUsize
pub struct UniformUsize { /* private fields */ }Expand description
The back-end implementing UniformSampler for usize.
Implementation notes
Sampling a usize value is usually used in relation to the length of an
array or other memory structure, thus it is reasonable to assume that the
vast majority of use-cases will have a maximum size under u32::MAX.
In part to optimise for this use-case, but mostly to ensure that results
are portable across 32-bit and 64-bit architectures (as far as is possible),
this implementation will use 32-bit sampling when possible.